/*
 Test BME-280 and send to LCD
 Jed Margolin  11/20/2021

 Read the BME280 and send the temperature and humidity to the base display
 Also works with BMP-280 but no humidity

 Uses I2C Interface for the BME280
 Displays the Pressure, Temperature, Humidity, and Device ID on the LCD

 Use SPI Display Board
 Use MSP430G2402

 Compiler Optimization must be set to 0-Register Optimization only

 Port 1:
   D0 input  not used
   D1 input  PB1
   D2 input  PB2
   D3 output error flag
   D4 output /CS not used
   D5 output SCLK
   D6 output SDOUT
   D7 output SDA

 Port 2:
   D0  output    LCD D4
   D1  output    LCD D5
   D2  output    LCD D6
   D3  output    LCD D7
   D4  output    LCD /E (Enable)
   D5  output    LCD RS (Register Select)
   D6  input     not used (XIN)
   D7  input     not used (XOUT)
*/
//==================================================================
#define MSPTYPE 0   // 1 = MSP430G2553, 0 = MSP430G2402 or other
#define TESTLOOP 0

#include <msp430.h>
#include "main.h"
#include "bme280.h"

int main(void)
{
    int n;

    WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer

// No crystal. Use the factory default values
    DCOCTL = CALDCO_1MHZ;      // factory calibration vqlues
    BCSCTL1 = CALBC1_1MHZ;
    BCSCTL2 = 0x06;            // DIVS (Divider for SMCLK) /8

// Port 1 is all digital I/O
    P1SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P1SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 1 - inputs
    P1REN |= (BIT0|BIT1|BIT2);    // pullup/pulldown enabled
    P1OUT |= (BIT0|BIT1|BIT2);    //
    P1DIR &= ~(BIT0|BIT1|BIT2);   // In

// Port 1 - outputs
    P1DIR |= (BIT3|BIT4|BIT5|BIT6|BIT7);
    P1OUT &= (BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 is all digital I/O
// Since we are not using a crystal XIN is P2.6 and XOUT is P2.7
    P2SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P2SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 - outputs for LCD
    P2DIR |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5);     //Out
    P2OUT &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5);

// Port 2 - inputs
    P2REN |= (BIT6|BIT7);    // pullup/pulldown enabled
    P2OUT |= (BIT6|BIT7);    //
    P2DIR &= ~(BIT6|BIT7);   // In

#if MSPTYPE
// Port 3 - inputs (the 2553 die has 3 ports)
    P3SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);  // In
    P3SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P3REN |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    // pullup/pulldown enabled
    P3OUT |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    //
    P3DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);   // In
#endif

    lcd_init();     // initialize lcd

    device_id = i2c_read(0xD0);
    asm(" nop");    // for debugger

    bme_init(); // Initialize bmp
    getcal();      // get calibration data for temp, pressure, and humidity and make digs

    for(n=32; n>0; n--)     // let it average to settle down first
    {
       read_bme280();
    }

    show_temperature = T;       // for testing the sensor  2430 is 24.3 degrees C is 75.7 degrees F
    show_humidity = Humidity3;  // for testing the sensor  231 is 23.1%
    asm(" nop");    // for debugger

    lcd_clear(0x80);      // clear the line starting at 0x80 (Line 1)
    lcd_clear(0xC0);      // clear the line starting at 0xC0 (Line 2)

    lcd_display();
    asm(" nop");    // for debugger

#if TESTLOOP
    temperature = 104;
// Loop for testing
   while (1)
   {

       P2OUT ^= BIT2;
       waitms(100);

   }  // end while test loop
#endif

// Set up timer Interrupts - ms
    CCTL0 = CCIE;                    // CCR0 interrupt enabled
    CCR0 = 31600;                    // 1 second interrupts (15800 = 500ms)
    TACTL = TASSEL_2 | ID_2 | MC_1;   // SMCLK, contmode = count up to CCR0
    TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction

    __bis_SR_register(LPM0_bits + GIE);       // Timer interrupts with LPM0 (Sleep until the next interrupt)

// It should never get here
    while (1)           // main loop
    {
       P1OUT ^= BIT3;   // error
    }  // end while main loop

}  // end main

//==========================================================
// Timer A0 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) Timer_A (void)
#else
#error Compiler not supported!
#endif
{
   if(irq_count == 1)
   {
      lcd_display();
   }

   if(irq_count-- == 0)
   {
       read_bme280();       // update the readings
       irq_count = 1;
   }

   asm(" nop");    // for debugger
} // end irq
//==========================================================
void wait5ms(void)     // 5ms
{
   volatile unsigned int i,j;

   for(i=41; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait29ms(void)     // 29ms
{
   volatile unsigned int i,j;

   for(i=241; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void waitseconds(unsigned int n)
{
    volatile unsigned int i,j,k;

    for(k=n; k>0; k--)
    {
       for(i=8200; i>0; i--)
       {
          for(j=10; j>0; j--);
       }
    }
    return;
}
//==============================================
